==> f-engine-0.0.4.tgz: package/es/canvas/index.js <==
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { isFunction } from '@antv/util';
import equal from './equal';
import { Text, Canvas as GCanvas } from '@antv/g-lite';
import { createMobileCanvasElement } from '@antv/g-mobile-canvas-element';
import { Renderer as CanvasRenderer } from '@antv/g-mobile-canvas';
import { createUpdater } from '../component/updater';
import EE from '@antv/event-emitter';
import Theme from './theme';
import Layout from './layout';
import { px2hd as defaultPx2hd, checkCSSRule, batch2hd } from './util';
import Gesture from '../gesture';
import { render as _render, updateComponents as _updateComponents } from './render';
function measureText(container, px2hd, theme) {
  return function (text, font) {
    var _ref = font || {},
      _ref$fontSize = _ref.fontSize,
      fontSize = _ref$fontSize === void 0 ? theme.fontSize : _ref$fontSize,
      _ref$fontFamily = _ref.fontFamily,
      fontFamily = _ref$fontFamily === void 0 ? theme.fontFamily : _ref$fontFamily,
      _ref$fontWeight = _ref.fontWeight,
      fontWeight = _ref$fontWeight === void 0 ? theme.fontWeight : _ref$fontWeight,
      _ref$fontVariant = _ref.fontVariant,
      fontVariant = _ref$fontVariant === void 0 ? theme.fontVariant : _ref$fontVariant,
      _ref$fontStyle = _ref.fontStyle,
      fontStyle = _ref$fontStyle === void 0 ? theme.fontStyle : _ref$fontStyle,
      _ref$textAlign = _ref.textAlign,
      textAlign = _ref$textAlign === void 0 ? theme.textAlign : _ref$textAlign,
      _ref$textBaseline = _ref.textBaseline,
      textBaseline = _ref$textBaseline === void 0 ? theme.textBaseline : _ref$textBaseline,
      _ref$lineWidth = _ref.lineWidth,
      lineWidth = _ref$lineWidth === void 0 ? 1 : _ref$lineWidth;
    var style = {
      x: 0,
      y: 0,
      fontSize: px2hd(fontSize),
      fontFamily: fontFamily,
      fontStyle: fontStyle,
      fontWeight: fontWeight,
      fontVariant: fontVariant,
      text: text,
      textAlign: textAlign,
      textBaseline: textBaseline,
      lineWidth: lineWidth,
      visibility: 'hidden'
    };
    var result = checkCSSRule('text', style);
    // @ts-ignore
    var shape = new Text({
      style: result
    });
    container.appendChild(shape);
    var _shape$getBBox = shape.getBBox(),
      width = _shape$getBBox.width,
      height = _shape$getBBox.height;
    shape.remove();
    return {
      width: width,
      height: height
    };
  };
}
// 顶层Canvas标签
var Canvas = /*#__PURE__*/function () {
  function Canvas(props) {
    _classCallCheck(this, Canvas);
    var context = props.context,
      _props$renderer = props.renderer,
      renderer = _props$renderer === void 0 ? new CanvasRenderer() : _props$renderer,
      width = props.width,
      height = props.height,
      customTheme = props.theme,
      customPx2hd = props.px2hd,
      customPixelRatio = props.pixelRatio,
      createImage = props.createImage,
      landscape = props.landscape,
      rendererContainer = props.container,
      customStyle = props.style,
      _props$animate = props.animate,
      animate = _props$animate === void 0 ? true : _props$animate;
    var px2hd = isFunction(customPx2hd) ? batch2hd(customPx2hd) : defaultPx2hd;
    // 初始化主题
    var theme = px2hd(_objectSpread(_objectSpread({}, Theme.getTheme()), customTheme));
    var pixelRatio = theme.pixelRatio,
      fontSize = theme.fontSize,
      fontFamily = theme.fontFamily,
      padding = theme.padding;
    var devicePixelRatio = customPixelRatio ? customPixelRatio : pixelRatio;
    // 组件更新器
    var updater = createUpdater(this);
    var canvasElement = createMobileCanvasElement(context);
    var canvas = new GCanvas({
      container: rendererContainer,
      canvas: canvasElement,
      devicePixelRatio: devicePixelRatio,
      renderer: renderer,
      width: width,
      height: height,
      supportTouchEvent: true,
      supportsPointerEvents: true,
      createImage: createImage
    });
    // 设置默认的全局样式
    var documentElement = canvas.document.documentElement;
    documentElement.setAttribute('fontSize', fontSize);
    documentElement.setAttribute('fontFamily', fontFamily);
    var container = canvas.getRoot();
    var _canvas$getConfig = canvas.getConfig(),
      canvasWidth = _canvas$getConfig.width,
      canvasHeight = _canvas$getConfig.height;
    var style = px2hd(_objectSpread({
      left: 0,
      top: 0,
      width: canvasWidth,
      height: canvasHeight,
      padding: padding
    }, customStyle));
    var layout = Layout.fromStyle(style);
    var gesture = new Gesture(canvas);
    // 供全局使用的一些变量
    var componentContext = {
      root: this,
      canvas: canvas,
      left: layout.left,
      top: layout.top,
      width: layout.width,
      height: layout.height,
      px2hd: px2hd,
      theme: theme,
      gesture: gesture,
      measureText: measureText(container, px2hd, theme)
    };
    var vNode = {
      key: undefined,
      style: {
        left: layout.left,
        top: layout.top,
        width: layout.width,
        height: layout.height
      },
      layout: {
        width: layout.width,
        height: layout.height,
        left: layout.left,
        top: layout.top,
        right: 0,
        bottom: 0
      },
      // @ts-ignore
      type: Canvas,
      props: props,
      shape: container,
      animate: animate,
      // @ts-ignore
      component: this,
      context: componentContext,
      updater: updater
    };
    this._ee = new EE();
    this.props = props;
    this.context = componentContext;
    this.updater = updater;
    this.gesture = gesture;
    this.theme = theme;
    this.canvas = canvas;
    this.container = container;
    this.canvasElement = canvasElement;
    this.vNode = vNode;
    // todo: 横屏事件逻辑
    this.landscape = landscape;
  }
  _createClass(Canvas, [{
    key: "updateComponents",
    value: function updateComponents(components) {
      _updateComponents(components);
    }
  }, {
    key: "update",
    value: function () {
      var _update = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(nextProps) {
        var props, vNode, _props$animate2, animate;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                props = this.props, vNode = this.vNode;
                if (!equal(nextProps, props)) {
                  _context.next = 3;
                  break;
                }
                return _context.abrupt("return");
              case 3:
                _props$animate2 = props.animate, animate = _props$animate2 === void 0 ? true : _props$animate2;
                this.props = nextProps;
                vNode.props = nextProps;
                vNode.animate = animate;
                _context.next = 9;
                return this.render();
              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function update(_x) {
        return _update.apply(this, arguments);
      }
      return update;
    }()
  }, {
    key: "render",
    value: function () {
      var _render2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var canvas, vNode;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                canvas = this.canvas, vNode = this.vNode;
                _context2.next = 3;
                return canvas.ready;
              case 3:
                _render(vNode);
              case 4:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function render() {
        return _render2.apply(this, arguments);
      }
      return render;
    }()
  }, {
    key: "emit",
    value: function emit(type, event) {
      this._ee.emit(type, event);
    }
  }, {
    key: "on",
    value: function on(type, listener) {
      this._ee.on(type, listener);
    }
  }, {
    key: "off",
    value: function off(type, listener) {
      this._ee.off(type, listener);
    }
  }, {
    key: "getCanvasEl",
    value: function getCanvasEl() {
      return this.canvasElement;
    }
  }, {
    key: "resize",
    value: function resize(width, height) {
      var canvas = this.canvas;
      canvas.resize(width, height);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var canvas = this.canvas;
      // 销毁也需要等 ready
      canvas.ready.then(function () {
        canvas.destroy();
      });
      this.props = null;
      this.context = null;
      this.updater = null;
      this.theme = null;
      this.canvas = null;
      this.container = null;
      this.canvasElement = null;
      this.vNode = null;
    }
  }]);
  return Canvas;
}();
export default Canvas;

==> f-engine-0.0.10.tgz: package/es/canvas/index.js <==
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { isFunction } from '@antv/util';
import equal from './equal';
import { Text, Canvas as GCanvas } from '@antv/g-lite';
import { createMobileCanvasElement } from '@antv/g-mobile-canvas-element';
import { Renderer as CanvasRenderer } from '@antv/g-mobile-canvas';
import { createUpdater } from '../component/updater';
import EE from '@antv/event-emitter';
import Theme from './theme';
import { px2hd as defaultPx2hd, checkCSSRule, batch2hd } from './util';
import Gesture from '../gesture';
import { render as _render, updateComponents as _updateComponents } from './render';
import { ClassComponent } from './workTags';
function measureText(container, px2hd, theme) {
  return function (text, font) {
    var _ref = font || {},
      _ref$fontSize = _ref.fontSize,
      fontSize = _ref$fontSize === void 0 ? theme.fontSize : _ref$fontSize,
      _ref$fontFamily = _ref.fontFamily,
      fontFamily = _ref$fontFamily === void 0 ? theme.fontFamily : _ref$fontFamily,
      _ref$fontWeight = _ref.fontWeight,
      fontWeight = _ref$fontWeight === void 0 ? theme.fontWeight : _ref$fontWeight,
      _ref$fontVariant = _ref.fontVariant,
      fontVariant = _ref$fontVariant === void 0 ? theme.fontVariant : _ref$fontVariant,
      _ref$fontStyle = _ref.fontStyle,
      fontStyle = _ref$fontStyle === void 0 ? theme.fontStyle : _ref$fontStyle,
      _ref$textAlign = _ref.textAlign,
      textAlign = _ref$textAlign === void 0 ? theme.textAlign : _ref$textAlign,
      _ref$textBaseline = _ref.textBaseline,
      textBaseline = _ref$textBaseline === void 0 ? theme.textBaseline : _ref$textBaseline,
      _ref$lineWidth = _ref.lineWidth,
      lineWidth = _ref$lineWidth === void 0 ? 1 : _ref$lineWidth;
    var style = {
      x: 0,
      y: 0,
      fontSize: px2hd(fontSize),
      fontFamily: fontFamily,
      fontStyle: fontStyle,
      fontWeight: fontWeight,
      fontVariant: fontVariant,
      text: text,
      textAlign: textAlign,
      textBaseline: textBaseline,
      lineWidth: lineWidth,
      visibility: 'hidden'
    };
    var result = checkCSSRule('text', style);
    var shape = new Text({
      style: result
    });
    container.appendChild(shape);
    var _shape$getBBox = shape.getBBox(),
      width = _shape$getBBox.width,
      height = _shape$getBBox.height;
    shape.remove();
    return {
      width: width,
      height: height
    };
  };
}
function computeLayout(style) {
  var left = style.left,
    top = style.top,
    width = style.width,
    height = style.height,
    padding = style.padding;
  var _padding = _slicedToArray(padding, 4),
    paddingTop = _padding[0],
    paddingRight = _padding[1],
    paddingBottom = _padding[2],
    paddingLeft = _padding[3];
  return {
    left: left + paddingLeft,
    top: top + paddingTop,
    width: width - paddingLeft - paddingRight,
    height: height - paddingTop - paddingBottom
  };
}
// 顶层Canvas标签
var Canvas = /*#__PURE__*/function () {
  function Canvas(props) {
    _classCallCheck(this, Canvas);
    var context = props.context,
      _props$renderer = props.renderer,
      renderer = _props$renderer === void 0 ? new CanvasRenderer() : _props$renderer,
      width = props.width,
      height = props.height,
      customTheme = props.theme,
      customPx2hd = props.px2hd,
      customPixelRatio = props.pixelRatio,
      createImage = props.createImage,
      landscape = props.landscape,
      rendererContainer = props.container,
      customStyle = props.style,
      _props$animate = props.animate,
      animate = _props$animate === void 0 ? true : _props$animate;
    var px2hd = isFunction(customPx2hd) ? batch2hd(customPx2hd) : defaultPx2hd;
    // 初始化主题
    var theme = px2hd(_objectSpread(_objectSpread({}, Theme), customTheme));
    var pixelRatio = theme.pixelRatio,
      fontSize = theme.fontSize,
      fontFamily = theme.fontFamily,
      padding = theme.padding;
    var devicePixelRatio = customPixelRatio ? customPixelRatio : pixelRatio;
    // 组件更新器
    var updater = createUpdater(this);
    var canvasElement = createMobileCanvasElement(context);
    var canvas = new GCanvas({
      container: rendererContainer,
      canvas: canvasElement,
      devicePixelRatio: devicePixelRatio,
      renderer: renderer,
      width: width,
      height: height,
      supportsTouchEvents: true,
      supportsPointerEvents: true,
      createImage: createImage
    });
    var container = canvas.getRoot();
    var _canvas$getConfig = canvas.getConfig(),
      canvasWidth = _canvas$getConfig.width,
      canvasHeight = _canvas$getConfig.height;
    var style = px2hd(_objectSpread({
      left: 0,
      top: 0,
      width: canvasWidth,
      height: canvasHeight,
      padding: padding
    }, customStyle));
    var layout = computeLayout(style);
    // 设置默认的全局样式
    var documentElement = canvas.document.documentElement;
    documentElement.setAttribute('fontSize', fontSize);
    documentElement.setAttribute('fontFamily', fontFamily);
    // 设置 container 的位置
    container.setAttribute('x', layout.left);
    container.setAttribute('y', layout.top);
    var gesture = new Gesture(canvas);
    // 供全局使用的一些变量
    var componentContext = {
      root: this,
      canvas: canvas,
      left: layout.left,
      top: layout.top,
      width: layout.width,
      height: layout.height,
      px2hd: px2hd,
      theme: theme,
      gesture: gesture,
      measureText: measureText(container, px2hd, theme)
    };
    var vNode = {
      key: undefined,
      tag: ClassComponent,
      style: layout,
      layout: layout,
      // @ts-ignore
      type: Canvas,
      props: props,
      shape: container,
      animate: animate,
      // @ts-ignore
      component: this,
      canvas: this,
      context: componentContext,
      updater: updater
    };
    this._ee = new EE();
    this.props = props;
    this.context = componentContext;
    this.updater = updater;
    this.gesture = gesture;
    this.theme = theme;
    this.canvas = canvas;
    this.container = container;
    this.canvasElement = canvasElement;
    this.vNode = vNode;
    // todo: 横屏事件逻辑
    this.landscape = landscape;
  }
  _createClass(Canvas, [{
    key: "updateComponents",
    value: function updateComponents(components) {
      _updateComponents(components);
    }
  }, {
    key: "update",
    value: function () {
      var _update = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(nextProps) {
        var props, vNode, _props$animate2, animate;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                props = this.props, vNode = this.vNode;
                if (!equal(nextProps, props)) {
                  _context.next = 3;
                  break;
                }
                return _context.abrupt("return");
              case 3:
                _props$animate2 = props.animate, animate = _props$animate2 === void 0 ? true : _props$animate2;
                this.props = nextProps;
                vNode.props = nextProps;
                vNode.animate = animate;
                _context.next = 9;
                return this.render();
              case 9:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
      function update(_x) {
        return _update.apply(this, arguments);
      }
      return update;
    }()
  }, {
    key: "render",
    value: function () {
      var _render2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        var canvas, vNode;
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                canvas = this.canvas, vNode = this.vNode;
                _context2.next = 3;
                return canvas.ready;
              case 3:
                _render(vNode);
              case 4:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
      function render() {
        return _render2.apply(this, arguments);
      }
      return render;
    }()
  }, {
    key: "emit",
    value: function emit(type, event) {
      this._ee.emit(type, event);
    }
  }, {
    key: "on",
    value: function on(type, listener) {
      this._ee.on(type, listener);
    }
  }, {
    key: "off",
    value: function off(type, listener) {
      this._ee.off(type, listener);
    }
  }, {
    key: "getCanvasEl",
    value: function getCanvasEl() {
      return this.canvasElement;
    }
  }, {
    key: "resize",
    value: function resize(width, height) {
      var canvas = this.canvas;
      canvas.resize(width, height);
    }
  }, {
    key: "toRawChildren",
    value: function toRawChildren(children) {
      return children;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var canvas = this.canvas;
      // 销毁也需要等 ready
      canvas.ready.then(function () {
        canvas.destroy();
      });
      this.props = null;
      this.context = null;
      this.updater = null;
      this.theme = null;
      this.canvas = null;
      this.container = null;
      this.canvasElement = null;
      this.vNode = null;
    }
  }]);
  return Canvas;
}();
export default Canvas;
